@@ -140,6 +140,9 @@ $cacti_version_codes = array(
 	'1.2.25'  => '0103',
 	'1.2.26'  => '0103',
 	'1.2.27'  => '0103',
+	'1.2.28'  => '0103',
+	'1.2.29'  => '0103',
+	'1.2.30'  => '0103',
 );
 
 $messages = array(
@@ -475,13 +478,16 @@ $phperrors = array (
 	E_USER_ERROR => 'USER_ERROR',
 	E_USER_WARNING => 'USER_WARNING',
 	E_USER_NOTICE  => 'USER_NOTICE',
-	E_STRICT => 'STRICT',
 	E_RECOVERABLE_ERROR  => 'RECOVERABLE_ERROR',
 	E_DEPRECATED => 'DEPRECATED',
 	E_USER_DEPRECATED  => 'USER_DEPRECATED',
 	E_ALL => 'ALL'
 );
 
+if (version_compare(PHP_VERSION, '8.4', '<')) {
+	$phperrors[E_STRICT] = 'STRICT';
+}
+
 if (cacti_version_compare(get_rrdtool_version(), '1.8.0', '>=')) {
 	$cdef_functions[] = 'ROUND';
 }
@@ -686,9 +692,10 @@ $availability_options = array(
 );
 
 $ping_methods = array(
-	PING_ICMP => __('ICMP Ping'),
-	PING_TCP  => __('TCP Ping'),
-	PING_UDP  => __('UDP Ping')
+	PING_ICMP        => __('ICMP Ping'),
+	PING_TCP         => __('TCP Ping'),
+	PING_TCP_CLOSED  => __('TCP Ping Closed'),
+	PING_UDP         => __('UDP Ping')
 );
 
 $logfile_verbosity = array(
@@ -1116,8 +1123,8 @@ if ((isset($_SESSION['sess_user_id']))) {
 		if (cacti_sizeof($consoles)) {
 			foreach ($consoles as $page) {
 				if (!$config['is_web'] || is_realm_allowed($page['id']+10000)) {
-					$menuname = (isset($page['extendedstyle']) && $page['extendedstyle'] != '' ? $page['extendedstyle'] : __('External Links'));
-					$menu[$menuname]['link.php?id=' . $page['id']] = $page['title'];
+					$menuname = (isset($page['extendedstyle']) && $page['extendedstyle'] != '' ? html_escape($page['extendedstyle']) : __('External Links'));
+					$menu[$menuname]['link.php?id=' . $page['id']] = html_escape($page['title']);
 				}
 			}
 		}
