@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -350,8 +350,6 @@ function html_graph_preview_filter($page, $action, $devices_where = '', $templat
 		</form>
 		<script type='text/javascript'>
 
-    	var refreshIsLogout = false;
-		var refreshMSeconds = <?php print read_user_setting('page_refresh')*1000;?>;
 		var graph_start     = <?php print get_current_graph_start();?>;
 		var graph_end       = <?php print get_current_graph_end();?>;
 		var timeOffset      = <?php print date('Z');?>;
@@ -405,6 +403,12 @@ function html_graph_preview_filter($page, $action, $devices_where = '', $templat
 		}
 
 		$(function() {
+			refreshIsLogout = false;
+			refreshIsLogout = false;
+			refreshMSeconds = <?php print read_user_setting('page_refresh')*1000;?>;
+
+			setupPageTimeout();
+
 			$('#go').off('click').on('click', function(event) {
 				event.preventDefault();
 				applyGraphFilter();
