@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  | Portions Copyright (C) 2010 Boris Lytochkin, Sponsored by Yandex LLC    |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
@@ -173,6 +173,7 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user = '',
 		}
 	} else {
 		$snmp_value = '';
+		$hostname = cacti_format_ipv6_colon($hostname);
 
 		/* net snmp want the timeout in seconds */
 		$timeout_s = (int) ceil($timeout_ms / 1000);
@@ -266,6 +267,7 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user =
 		}
 	} else {
 		$snmp_value = '';
+		$hostname = cacti_format_ipv6_colon($hostname);
 
 		/* net snmp want the timeout in seconds */
 		$timeout_s = (int) ceil($timeout_ms / 1000);
@@ -354,6 +356,7 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user =
 		}
 	} else {
 		$snmp_value = '';
+		$hostname = cacti_format_ipv6_colon($hostname);
 
 		/* net snmp want the timeout in seconds */
 		$timeout_s = (int) ceil($timeout_ms / 1000);
@@ -442,6 +445,14 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 		return array();
 	}
 
+	if (is_array($oid)) {
+		foreach($oid as $index => $o) {
+			$oid[$index] = trim($o);
+		}
+	} else {
+		$oid = trim($oid);
+	}
+
 	$session->value_output_format = $value_output_format;
 
 	if ($non_repeaters === NULL) {
@@ -457,7 +468,7 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 	}
 
 	try {
-		$out = @$session->walk(trim($oid), false, $max_repetitions, $non_repeaters);
+		$out = @$session->walk($oid, false, $max_repetitions, $non_repeaters);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -477,9 +488,17 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 	}
 
 	if (cacti_sizeof($out)) {
-		foreach($out as $oid => $value){
-			$out[$oid] = format_snmp_string($value, false, $value_output_format);
+		foreach($out as $oid => $value) {
+			if (is_array($value)) {
+				foreach($value as $index => $sval) {
+					$out[$oid][$index] = format_snmp_string($sval, false, $value_output_format);
+				}
+			} elseif ($out[$oid] !== false) {
+				$out[$oid] = format_snmp_string($value, false, $value_output_format);
+			}
 		}
+	} else {
+		$out = format_snmp_string($oid, false, $value_output_format);
 	}
 
 	return $out;
@@ -491,10 +510,16 @@ function cacti_snmp_session_get($session, $oid, $strip_alpha = false) {
 	if (is_array($oid) && cacti_sizeof($oid) == 0) {
 		cacti_log('Empty OID!', false);
 		return array();
+	} elseif (is_array($oid)) {
+		foreach($oid as $index => $o) {
+			$oid[$index] = trim($o);
+		}
+	} else {
+		$oid = trim($oid);
 	}
 
 	try {
-		$out = @$session->get(trim($oid));
+		$out = @$session->get($oid);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -507,11 +532,12 @@ function cacti_snmp_session_get($session, $oid, $strip_alpha = false) {
 		if ($session->getErrno() == SNMP::ERRNO_TIMEOUT) {
 			cacti_log('WARNING: SNMP Error:\'Timeout (' . round($info['timeout']/1000,0) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
+
 		return false;
 	}
 
 	if (is_array($out)) {
-		foreach($out as $oid => $value){
+		foreach($out as $oid => $value) {
 			$out[$oid] = format_snmp_string($value, false, SNMP_STRING_OUTPUT_GUESS, $strip_alpha);
 		}
 	} else {
@@ -528,8 +554,16 @@ function cacti_snmp_session_getnext($session, $oid) {
 		return array();
 	}
 
+	if (is_array($oid)) {
+		foreach($oid as $index => $o) {
+			$oid[$index] = trim($o);
+		}
+	} else {
+		$oid = trim($oid);
+	}
+
 	try {
-		$out = @$session->getnext(trim($oid));
+		$out = @$session->getnext($oid);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -545,7 +579,7 @@ function cacti_snmp_session_getnext($session, $oid) {
 	}
 
 	if (is_array($out)) {
-		foreach($out as $oid => $value){
+		foreach($out as $oid => $value) {
 			$out[$oid] = format_snmp_string($value, false);
 		}
 	} else {
@@ -555,6 +589,18 @@ function cacti_snmp_session_getnext($session, $oid) {
 	return $out;
 }
 
+function cacti_snmp_validate_oid($oid) {
+	$oid = ltrim($oid, '.');
+
+	$validate = array_unique(array_map('is_numeric', explode('.', $oid)));
+
+	if (array_search(false, $validate, true)) {
+		return false;
+	} else {
+		return true;
+	}
+}
+
 function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user = '', $auth_pass = '',
 	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '',
 	$port = 161, $timeout_ms = 500, $retries = 0, $bulk_walk_size = 10, $environ = 'SNMP',
@@ -631,6 +677,7 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user = '',
 	} else {
 		/* ucd/net snmp want the timeout in seconds */
 		$timeout_s = (int) ceil($timeout_ms / 1000);
+		$hostname = cacti_format_ipv6_colon($hostname);
 
 		if ($version == '1') {
 			$snmp_auth = '-c ' . snmp_escape_string($community); /* v1/v2 - community string */
@@ -687,11 +734,26 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user = '',
 				}
 			}
 
+			/**
+			 * Using this technique to catch multi-line
+			 * snmpwalk responses from net-snmp.  This happens
+			 * usually on sysDescr on Cisco devices.
+			 */
 			$i = 0;
+
 			foreach($temp_array as $index => $value) {
 				if (preg_match('/(.*) =.*/', $value)) {
-					$snmp_array[$i]['oid']   = trim(preg_replace('/(.*) =.*/', "\\1", $value));
-					$snmp_array[$i]['value'] = format_snmp_string($value, true, $value_output_format);
+					$parts   = explode('=', $value, 2);
+					$t_oid   = trim($parts[0]);
+					$t_value = $parts[1];
+
+					if (!cacti_snmp_validate_oid($t_oid)) {
+						cacti_log(sprintf('WARNING: SNMP Agent exploit attempted on SNMP agent from host ip: %s with oid: %s', $hostname, $t_oid), false, 'SECURITY');
+						continue;
+					}
+
+					$snmp_array[$i]['oid']   = $t_oid;
+					$snmp_array[$i]['value'] = $t_value;
 					$i++;
 				} else {
 					$snmp_array[$i-1]['value'] .= $value;
@@ -700,17 +762,30 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user = '',
 		}
 	}
 
+	/**
+	 * replay the array to escape value data in case of a multi-line exploit
+	 */
+	if (cacti_sizeof($snmp_array)) {
+		foreach($snmp_array as $index => $data) {
+			$snmp_array[$index]['value'] = format_snmp_string($data['value'], false, $value_output_format);
+		}
+	}
+
 	return $snmp_array;
 }
 
 function format_snmp_string($string, $snmp_oid_included, $value_output_format = SNMP_STRING_OUTPUT_GUESS, $strip_alpha = false) {
 	global $banned_snmp_strings;
 
+	if ($string === null) {
+		return '';
+	}
+
 	$string = preg_replace(REGEXP_SNMP_TRIM, '', trim($string));
 
 	if ($snmp_oid_included) {
 		/* strip off all leading junk (the oid and stuff) */
-		$string_array = explode('=', $string);
+		$string_array = explode('=', $string, 2);
 
 		if (cacti_sizeof($string_array) == 1) {
 			/* trim excess first */
@@ -807,7 +882,6 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 		 * and hex-string: from the passed value
 		 */
 		$output = '';
-
 		$parts  = explode(' ', $string);
 
 		if (cacti_sizeof($parts) == 4) {
@@ -854,12 +928,11 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 			$possible_ip = false;
 		}
 	} elseif (substr(strtolower($string), 0, 4) == 'hex:') {
-		$output = '';
-
 		/* strip off the 'Hex:' */
 		$string = trim(str_ireplace('hex:', '', $string));
 
 		/* normalize some forms */
+		$output = '';
 		$string = str_replace(array(' ', '-', '.'), ':', $string);
 		$parts  = explode(':', $string);
 
@@ -897,19 +970,18 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 function snmp_escape_string($string) {
 	global $config;
 
-	if (! defined('SNMP_ESCAPE_CHARACTER')) {
-		if ($config['cacti_server_os'] == 'win32') {
-			define('SNMP_ESCAPE_CHARACTER', '"');
-		} else {
-			define('SNMP_ESCAPE_CHARACTER', "'");
+	if (!defined('SNMP_ESCAPE_CHARACTER')) {
+		define('SNMP_ESCAPE_CHARACTER', '"');
+	}
+
+	if ($config['cacti_server_os'] == 'win32') {
+		if (substr_count($string, SNMP_ESCAPE_CHARACTER)) {
+			$string = str_replace(SNMP_ESCAPE_CHARACTER, "\\" . SNMP_ESCAPE_CHARACTER, $string);
+			return SNMP_ESCAPE_CHARACTER . $string . SNMP_ESCAPE_CHARACTER;
 		}
 	}
 
-	if (substr_count($string, SNMP_ESCAPE_CHARACTER)) {
-		$string = substr_replace(SNMP_ESCAPE_CHARACTER, "\\" . SNMP_ESCAPE_CHARACTER, $string);
-	}
-
-	return SNMP_ESCAPE_CHARACTER . $string . SNMP_ESCAPE_CHARACTER;
+	return cacti_escapeshellarg($string);
 }
 
 function snmp_get_method($type = 'walk', $version = 1, $context = '', $engineid = '',
