@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -51,6 +51,7 @@ $debug    = false;
 $force    = false;
 $archived = 0;
 $purged   = 0;
+$start    = microtime(true);
 
 if (cacti_sizeof($parms)) {
 	foreach ($parms as $parameter) {
@@ -89,8 +90,11 @@ if (cacti_sizeof($parms)) {
 maint_debug('Checking for Purge Actions');
 
 /* silently end if the registered process is still running, or process table missing */
-if (!register_process_start('maintenance', 'master', $config['poller_id'], read_config_option('maintenance_timeout'))) {
-	exit(0);
+if (!$force) {
+	if (!register_process_start('maintenance', 'master', $config['poller_id'], read_config_option('maintenance_timeout'))) {
+		maint_debug('Exiting due to another process running');
+		exit(0);
+	}
 }
 
 if ($config['poller_id'] == 1) {
@@ -99,6 +103,8 @@ if ($config['poller_id'] == 1) {
 	authcache_purge();
 
 	secpass_check_expired();
+
+	reindex_devices();
 }
 
 // Check the realtime cache and poller
@@ -110,10 +116,83 @@ api_device_purge_deleted_devices();
 // Rotate Cacti Logs
 logrotate_check($force);
 
-unregister_process('maintenance', 'master', $config['poller_id']);
+// Remove deleted devices
+remove_aged_row_cache();
+
+if ($config['poller_id'] > 1) {
+	api_plugin_hook('poller_remote_maint');
+}
+
+$end = microtime(true);
+
+cacti_log(sprintf('MAINT STATS: Time:%0.2f', $end - $start), false, 'SYSTEM');
+
+if (!$force) {
+	unregister_process('maintenance', 'master', $config['poller_id']);
+}
 
 exit(0);
 
+function reindex_devices() {
+	global $config;
+
+	$schedule = read_config_option('automatic_reindex');
+
+	// 0 - Disabled
+	// 1 - Daily at Midnight
+	// 2 - Weekly on Sunday
+	// 3 - Monthly on Sunday
+
+	$command_string = cacti_escapeshellcmd(read_config_option('path_php_binary'));
+	$extra_args     = $config['base_path'] . '/cli/poller_reindex_hosts.php --id=all --qid=all';
+
+	if ($schedule == 0) {
+		return;
+	}
+
+	$last_run = read_config_option('periodic_reindex_lastrun');
+	$now      = time();
+
+	if (empty($last_run)) {
+		set_config_option('periodic_reindex_lastrun', time());
+		return;
+	} else {
+		if ($schedule == 1) {
+			if (date('z', $now) != date('z', $last_run)) {
+				set_config_option('periodic_reindex_lastrun', $now);
+				exec_background($command_string, $extra_args);
+			}
+		} elseif ($schedule == 2) {
+			if (date('z', $now) != date('z', $last_run) && date('w', $now) == 0) {
+				exec_background($command_string, $extra_args);
+			}
+		} elseif ($schedule == 3) {
+			if (date('z', $now) != date('z', $last_run)) {
+				if (date('w', $now) == 0 && date('n', $now) != date('n', $last_run)) {
+					exec_background($command_string, $extra_args);
+				}
+			}
+		}
+	}
+}
+
+function remove_aged_row_cache() {
+	$classes = array_rekey(
+		db_fetch_assoc('SELECT REPLACE(name, "time_last_change_", "") AS name, value
+			FROM settings
+			WHERE name LIKE "time_last_change%"'),
+		'name', 'value'
+	);
+
+	if (cacti_sizeof($classes)) {
+		foreach($classes as $name => $ts) {
+			db_execute_prepared('DELETE FROM user_auth_row_cache
+				WHERE class = ? AND UNIX_TIMESTAMP(time) < ?',
+				array($name, $ts));
+		}
+	}
+}
+
 function logrotate_check($force) {
 	global $disable_log_rotation;
 
@@ -173,7 +252,7 @@ function rrdfile_purge($force) {
 	global $archived, $purged, $poller_start;
 
 	/* are my tables already present? */
-	$purge = db_fetch_cell('SELECT count(*)
+	$purge = db_fetch_cell('SELECT COUNT(*)
 		FROM data_source_purge_action');
 
 	/* if the table that holds the actions is present, work on it */
@@ -184,7 +263,7 @@ function rrdfile_purge($force) {
 		while (true) {
 			maint_debug('Grabbing 1000 RRDfiles to Remove');
 
-			$file_array = db_fetch_assoc('SELECT id, name, local_data_id, action
+			$file_array = db_fetch_assoc('SELECT DISTINCT id, name, local_data_id, action
 				FROM data_source_purge_action
 				ORDER BY name
 				LIMIT 1000');
@@ -200,18 +279,25 @@ function rrdfile_purge($force) {
 				if ($force) {
 					cleanup_ds_and_graphs();
 				}
+			} else {
+				maint_debug('No RRDfiles found for archiving or removal');
 			}
 		}
 
 		/* record the start time */
 		$poller_end = microtime(true);
+		set_config_option('rrdcleaner_last_run_time', time());
 		$string = sprintf('RRDMAINT STATS: Time:%4.4f Purged:%s Archived:%s', ($poller_end - $poller_start), $purged, $archived);
 		cacti_log($string, true, 'SYSTEM');
+	} else {
+		maint_debug('No RRDfiles scheduled for arching or removal');
 	}
 }
 
-/** realtime_purge_cache() - This function will purge files in the realtime directory
- *  that are older than 2 hours without changes */
+/**
+ * realtime_purge_cache() - This function will purge files in the realtime directory
+ * that are older than 2 hours without changes
+ */
 function realtime_purge_cache() {
 	/* remove all Realtime files over than 2 hours */
 	if (read_config_option('realtime_cache_path') != '') {
@@ -237,8 +323,7 @@ function realtime_purge_cache() {
 }
 
 /*
- * logrotate_rotatenow
- * Rotates the cacti log
+ * logrotate_rotatenow - Rotates the cacti log
  */
 function logrotate_rotatenow() {
 	global $config;
@@ -288,8 +373,8 @@ function logrotate_rotatenow() {
 	cacti_log($string, true, 'SYSTEM');
 }
 
-/* logrotate_file_rotate()
- * rotates the specified log file, appending date given
+/**
+ * logrotate_file_rotate() - rotates the specified log file, appending date given
  */
 function logrotate_file_rotate($name, $log, $date) {
 	if (empty($log)) {
@@ -337,9 +422,8 @@ function logrotate_file_rotate($name, $log, $date) {
 	return 0;
 }
 
-/*
- * logrotate_file_clean
- * Cleans up any old log files that should be removed
+/**
+ * logrotate_file_clean - Cleans up any old log files that should be removed
  */
 function logrotate_file_clean($name, $log, $date, $rotation) {
 	global $config;
@@ -398,15 +482,15 @@ function logrotate_file_clean($name, $log, $date, $rotation) {
 	clearstatcache();
 }
 
-/*
- * secpass_check_expired
- * Checks user accounts to determine if the accounts and/or their passwords should be expired
+/**
+ * secpass_check_expired - Checks user accounts to determine if the accounts and/or their passwords should be expired
  */
 function secpass_check_expired () {
 	maint_debug('Checking for Account / Password expiration');
 
 	// Expire Old Accounts
 	$e = read_config_option('secpass_expireaccount');
+
 	if ($e > 0 && is_numeric($e)) {
 		$t = time();
 		db_execute_prepared("UPDATE user_auth
@@ -426,6 +510,7 @@ function secpass_check_expired () {
 			AND id > 1",
 			array($t));
 	}
+
 	$e = read_config_option('secpass_expirepass');
 	if ($e > 0 && is_numeric($e)) {
 		$t = time();
@@ -448,8 +533,7 @@ function secpass_check_expired () {
 }
 
 /*
- * remove_files
- * remove all unwanted files; the list is given by table data_source_purge_action
+ * remove_files - remove all unwanted files; the list is given by table data_source_purge_action
  */
 function remove_files($file_array) {
 	global $config, $debug, $archived, $purged;
@@ -463,64 +547,80 @@ function remove_files($file_array) {
 		$rra_path = $config['base_path'] . '/rra';
 	}
 
-	if ( read_config_option('storage_location')) {
+	if (read_config_option('storage_location')) {
 		$rrdtool_pipe = rrd_init();
+
 		rrdtool_execute('setcnn timeout off', false, RRDTOOL_OUTPUT_NULL, $rrdtool_pipe, $logopt = 'POLLER');
-	}else {
+	} else {
 		/* let's prepare the archive directory */
 		$rrd_archive = read_config_option('rrd_archive', true);
+
 		if ($rrd_archive == '') {
 			$rrd_archive = $rra_path . '/archive';
 		}
+
 		rrdclean_create_path($rrd_archive);
 	}
 
 	/* now scan the files */
 	foreach ($file_array as $file) {
-		$source_file = $rra_path . '/' . $file['name'];
+		$real_file = str_replace('<path_rra>', $rra_path, $file['name']);
+		$real_file = str_replace('<path_cacti>', $config['base_path'], $real_file);
+		$base_file = str_replace('<path_rra>', '', $file['name']);
+		$base_file = str_replace('<path_cacti>', '', $base_file);
 
-		if( read_config_option('storage_location') == 0) {
+		if (read_config_option('storage_location') == 0) {
 			switch ($file['action']) {
 				case '1' :
-					if (unlink($source_file)) {
-						maint_debug('Deleted: ' . $file['name']);
-					} else {
-						cacti_log($file['name'] . " ERROR: RRDfile Maintenance unable to delete from $rra_path!", true, 'MAINT');
+					if (file_exists($real_file)) {
+						if (unlink($real_file)) {
+							maint_debug('Deleted: ' . $real_file);
+							$purged++;
+						} else {
+							cacti_log("WARNING: RRDfile Maintenance is unable to remove $real_file from $rra_path!", true, 'MAINT');
+						}
 					}
-					$purged++;
+
 					break;
 				case '3' :
-					$target_file = $rrd_archive . '/' . $file['name'];
+					$target_file = $rrd_archive . '/' . $base_file;
 					$target_dir = dirname($target_file);
 					if (!is_dir($target_dir)) {
 						rrdclean_create_path($target_dir);
 					}
 
-					if (rename($source_file, $target_file)) {
-						maint_debug('Moved: ' . $file['name'] . ' to: ' . $rrd_archive);
-					} else {
-						cacti_log($file['name'] . " ERROR: RRDfile Maintenance unable to move to $rrd_archive!", true, 'MAINT');
+					if (file_exists($real_file)) {
+						if (rename($real_file, $target_file)) {
+							maint_debug("Moved: $real_file to: $target_file");
+							$archived++;
+						} else {
+							cacti_log("WARNING: RRDfile Maintenance is unable to move $real_file to $target_file!", true, 'MAINT');
+						}
 					}
-					$archived++;
+
 					break;
 			}
-		}else {
+		} else {
 			switch($file['action']) {
 				case '1':
-					if ( rrdtool_execute('unlink ' . $source_file, false, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, $logopt = 'MAINT')) {
+					if (rrdtool_execute('unlink ' . $file['name'], false, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, $logopt = 'MAINT')) {
 						maint_debug('Deleted: ' . $file['name']);
-					}else {
-						cacti_log($file['name'] . 'ERROR: RRDfile Maintenance unable to delete from RRDproxy!', true, 'MAINT');
+					} else {
+						cacti_log("WARNING RRDfile Maintenance is unable to remove {$file['name']} from the RRDproxy!", true, 'MAINT');
 					}
+
 					$purged++;
+
 					break;
 				case '3':
-					if ( rrdtool_execute('archive ' . $source_file, false, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, $logopt = 'MAINT')) {
-						maint_debug('Moved: ' . $file['name'] . ' to: RRDproxy Archive');
-					}else {
-						cacti_log($file['name'] . 'ERROR: RRDfile Maintenance unable to move to RRDproxy Archive!', true, 'MAINT');
+					if (rrdtool_execute('archive ' . $file['name'], false, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, $logopt = 'MAINT')) {
+						maint_debug("Moved: {file['name']} to: RRDproxy Archive");
+					} else {
+						cacti_log("WARNING RRDfile Maintenance is unable to move {$file['name']} to the RRDproxy Archive!", true, 'MAINT');
 					}
+
 					$archived++;
+
 					break;
 			}
 		}
@@ -567,7 +667,7 @@ function remove_files($file_array) {
 		}
 	}
 
-	if(read_config_option('storage_location')) {
+	if (read_config_option('storage_location')) {
 		rrd_close($rrdtool_pipe);
 	}
 
@@ -578,7 +678,7 @@ function rrdclean_create_path($path) {
 	global $config;
 
 	if (!is_dir($path)) {
-		if (mkdir($path, 0775)) {
+		if (mkdir($path, 0775, true)) {
 			if ($config['cacti_server_os'] != 'win32') {
 				$owner_id      = fileowner($config['rra_path']);
 				$group_id      = filegroup($config['rra_path']);
@@ -627,7 +727,7 @@ function cleanup_ds_and_graphs() {
 		}
 	}
 
-	if (empty ($remove_ldis)) {
+	if (empty($remove_ldis)) {
 		maint_debug('No missing rrd files found');
 		return 0;
 	}
@@ -664,19 +764,20 @@ function maint_debug($message) {
 	global $debug;
 
 	if ($debug) {
-		print trim($message) . "\n";
+		print "DEBUG: " . trim($message) . "\n";
 	}
 }
 
-/*  display_version - displays version information */
+/**
+ * display_version - displays version information
+ */
 function display_version() {
 	$version = get_cacti_version();
 	print "Cacti Maintenance Poller, Version $version, " . COPYRIGHT_YEARS . "\n";
 }
 
-/*
- * display_help
- * displays the usage of the function
+/**
+ * display_help - displays the usage of the function
  */
 function display_help() {
 	display_version();
